using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

namespace DrawingObjects
{
	public class Glyph {
		protected Rectangle bounds;    // local coordinates
		protected Matrix    transform = null;
		protected bool      selected = false;
		protected Scene     scene;

		/*********************************
		 * PUBLIC Implementation
		 ********************************/
		public Glyph(Scene scene) {
			bounds = new Rectangle();
			this.scene = scene;
		}

		public void Repaint() {
			scene.Repaint();
		}

		// Determines if this glyph contains the point (x, y) specified int global coordinates
		public bool Contains(int x, int y) {
			Point[] pts = {new Point(x, y)};

			if (transform != null) {
				if (transform.IsInvertible) {
					Matrix inverseTransform = transform.Clone();
					inverseTransform.Invert();
					// This converts the point from global to local coordinates
					inverseTransform.TransformPoints(pts);
				}
			}

			return bounds.Contains(pts[0]);
		}

		public Matrix Transform {
			get { return transform; }
			set {
				transform = value;
				Repaint();
			}
		}

		public bool Selected {
			get { return selected; }
			set {
				selected = value;
				Repaint();
			}
		}

		public Rectangle Bounds {
			get { return bounds; }
		}

		/*********************************
		 * Internal Implementation
		 ********************************/

		// Don't override this, instead override paintGlyph()
		internal void Paint(Graphics g) {
			Matrix origTransform = null;
			if (transform != null) {
				origTransform = g.Transform;
				g.Transform = transform;
			}

			PaintGlyph(g);

			if (selected) {
				g.DrawRectangle(scene.SelectionPen, bounds);
			}

			if (transform != null) {
				g.Transform = origTransform;
			}
		}

		// Override this one to control how a glyph is painted
		public virtual void PaintGlyph(Graphics g) {
		}

		// Override this one to specify how big a glyph is
		public virtual void ComputeBounds() {
		}
	}
}