using System;
using System.Drawing;
using System.Drawing.Drawing2D;

namespace DrawingObjects {
	public class RectGlyph : Glyph {
		protected Rectangle rect;
		protected Brush brush;
		protected Pen pen;

		public RectGlyph(Scene scene, int x, int y, int width, int height) : base(scene) {
			pen = new Pen(Color.Black, 0);
			brush = null;
			Rectangle = new Rectangle(x, y, width, height);
		}

		public Brush Brush {
			get { return brush; }
			set { 
				brush = value;
				Repaint();
			}
		}

		public Pen Pen {
			get { return pen; }
			set {
				pen = value;
				ComputeBounds();  // Necessary because pen width could change
				Repaint();
			}
		}

		public Rectangle Rectangle {
			get { return bounds; }
			set {
				rect = value;
				ComputeBounds();
				Repaint();
			}
		}

		public override void ComputeBounds() {
			bounds = new Rectangle(rect.X, rect.Y, rect.Width, rect.Height);
			if (pen != null) {
				bounds.Inflate((int)Math.Ceiling((pen.Width / 2) + 0.5f), (int)Math.Ceiling((pen.Width / 2) + 0.5f));
			}
		}

		public override void PaintGlyph(Graphics g) {
			if (brush != null) {
				g.FillRectangle(brush, rect.X, rect.Y, rect.Width, rect.Height);
			}
			if (pen != null) {
				g.DrawRectangle(pen, rect.X, rect.Y, rect.Width, rect.Height);
			}
		}
	}
}